<?php

class ImageThumb {

    // публичные свойства, которые можно и нужно устанавливать либо здесь,
    // либо после создания объекта (например, $obj->thumbType = IMG_JPG)

    // путь к каталогу, куда складываются картинки
    public $saveDir = './thumbs';

    // тип изображения на выходе (php-константы IMG_*)
    public $thumbType = IMG_PNG;

    // для PNG и JPG - качество изображения (от 0 до 100)
    public $quality = 60;

    // настройка фильтра: число от 0 до 100.
    // Чем больше число, тем интенсивнее применяется фильтр.
    // "Нормально" порядка 70...80
    //public $filterIntensity = 80;
	//public $filterIntensity = 100;
	public $filterIntensity = 10;

    // настройка фильтра: отрицательное число от -255 до 0.
    // Чем больше модуль числа, тем чаще пиксели затемняются
    // "Нормально" порядка -50...-20
    //public $filterDark = -30;
	public $filterDark = -50;

    // настройка фильтра: положительное число от 0 до 255
    // Чем больше модуль числа, тем чаще пиксели засветляются
    // "Нормально" порядка 20...50
    //public $filterBright = 30;
	public $filterBright = 50;

    // !!! дальше нигде ничего не меняем !!!

    private $originalImg = null;
    private $originalWidth = 0;
    private $originalHeight = 0;

    private $croppedImg = null;
    private $croppedWidth = 0;
    private $croppedHeight = 0;

    private $thumbImg = null;
    private $thumbWidth = 0;
    private $thumbHeight = 0;

    private $mimeType = '';


    public function makeThumb($url, $crop_x, $crop_y, $thumb_w, $thumb_h) {
        if ($this->saveDir && ! is_dir($this->saveDir)) {
            die('Dir not exists: ' . $this->saveDir);
        }
        switch($this->thumbType) {
        }
        $this->init();
        if (! $this->createOriginalImage($url)) {
            return false;
        }
        $this->createCroppedImage($crop_x, $crop_y);
        $this->createThumbImage($thumb_w, $thumb_h);
		/*
		if($apply_mirror) $this->createMirrorImage();
		
        if ($apply_filter) {
            $this->applyFilter();
        }
		*/
        ob_start();
        $this->outputThumb();
        $img_str = ob_get_clean();
        imagedestroy($this->originalImg);
        imagedestroy($this->croppedImg);
        imagedestroy($this->thumbImg);
        $save_to = $this->getSavePath($url);
        file_put_contents($save_to, $img_str);


                                        return [
            'file' => $save_to,
        ];
               
    }

    private function init() {
        $this->originalImg = null;
        $this->originalWidth = 0;
        $this->originalHeight = 0;
        $this->croppedImg = null;
        $this->croppedWidth = 0;
        $this->croppedHeight = 0;
        $this->thumbImg = null;
    }

    private function getSavePath($url) {
        $filename = basename($url);
        $ext = pathinfo($url, PATHINFO_EXTENSION);
        $name = pathinfo($url, PATHINFO_FILENAME);
        return rtrim($this->saveDir, '/') . '/' . $name . '-thumb.' . $ext;
    }

    private function createOriginalImage($url) {
        $data = @getimagesize($url);
        if ($data === false) {
            return false;
        }
        $this->originalWidth  = $data[0];
        $this->originalHeight = $data[1];
        switch($data[2]) {
            case IMG_GIF:
                $this->originalImg = imagecreatefromgif($url);
                break;
            case IMG_JPG:
                $this->originalImg = imagecreatefromjpeg($url);
                break;
            case IMG_PNG:
                $this->originalImg = imagecreatefrompng($url);
                break;
            default:
                return false;
        }
        return true;
    }

    private function createCroppedImage($crop_x, $crop_y) {
        $margin_x = round($crop_x * $this->originalWidth  / 100);
        $margin_y = round($crop_y * $this->originalHeight / 100);
        $this->croppedWidth  = $this->originalWidth  - $margin_x * 2;
        $this->croppedHeight = $this->originalHeight - $margin_y * 2;
        $this->croppedImg = imagecreatetruecolor($this->croppedWidth, $this->croppedHeight);
		
		//imagecolorallocatealpha($this->croppedImg,0xFF,0xFF,0xFF,0);
		$white = imagecolorallocatealpha($this->croppedImg,0xFF,0xFF,0xFF,0);
        imagefill($this->croppedImg, 0, 0, $white);	
	
        // сохранение прозрачности (для PNG и GIF)
        imagealphablending($this->croppedImg, false);
        imagesavealpha($this->croppedImg, true);
        imagecopy($this->croppedImg, $this->originalImg, 0, 0, $margin_x, $margin_y, $this->croppedWidth, $this->croppedHeight);
    }

    private function createThumbImage($thumb_w, $thumb_h) {
        $this->thumbWidth = $thumb_w;
        $this->thumbHeight = $thumb_h;
        $x_ratio = $this->croppedWidth / $this->thumbWidth;
        $y_ratio = $this->croppedHeight / $this->thumbHeight;
        if (abs($x_ratio - 1) < abs($y_ratio - 1)) {
            $src_w = $this->croppedWidth;
            $src_h = round($this->croppedWidth * $this->thumbHeight / $this->thumbWidth);
            $src_x = 0;
            $src_y = round(($this->croppedHeight - $src_h) / 2);
        } else {
            $src_w = round($this->croppedHeight * $this->thumbWidth / $this->thumbHeight);
            $src_h = $this->croppedHeight;
            $src_x = round(($this->croppedWidth - $src_w) / 2);
            $src_y = 0;
        }
        $this->thumbImg = imagecreatetruecolor($this->thumbWidth, $this->thumbHeight);
		
		//imagecolorallocatealpha($this->thumbImg,0xFF,0xFF,0xFF,0);
		$white = imagecolorallocatealpha($this->thumbImg,0xFF,0xFF,0xFF,0);
        imagefill($this->thumbImg, 0, 0, $white);
		imagefill($this->croppedImg, 0, 0, $white);

		//$this->thumbImg = $this->croppedImg;
		
		imagecolortransparent($this->thumbImg, $white);
		/*
        imagecopyresampled(
            $this->thumbImg, $this->croppedImg,
            0, 0, $src_x, $src_x,
            //$this->thumbWidth, $this->thumbHeight, $src_w, $src_h
			$this->thumbWidth, $this->thumbHeight,
			 $src_w, $src_h
			// $src_w, $src_h
			//max($src_w,$this->thumbWidth), max($src_h,$this->thumbHeight)
        );
		*/
        imagecopyresampled(
            $this->thumbImg, $this->croppedImg,
            0, 0, $src_x, $src_y,
            $this->thumbWidth, $this->thumbHeight, $src_w, $src_h
        );
		
		echo $this->thumbImg." ". $this->croppedImg." ".
            "0"." ". "0"." ". $src_x." ". $src_y." ".
            //$this->thumbWidth, $this->thumbHeight,
			max($src_w,$this->thumbWidth)." ". max($src_h,$this->thumbHeight);
			
		echo "<hr>".$this->thumbImg." ". $this->croppedImg." ".
            "0"." ". "0"." ". max($src_x,0)." ". max($src_x,0)." ".
            $this->thumbWidth." ". $this->thumbHeight." ".
			$src_w." ". $src_h;
		/*
		imagecopy(
			$this->thumbImg, $this->croppedImg,
            0, 0, max($src_x,0), max($src_x,0),
            //$this->thumbWidth, $this->thumbHeight,
			max($src_x,0)$src_w, $src_h
        );
		*/
		
			
    }
	

    private function outputThumb() {
				
				$img = $this->thumbImg;

                $this->mimeType = 'image/jpeg';
                imagejpeg($img, null, $this->quality);
		
		/*
        switch($this->thumbType) {
            case IMG_GIF:
                $this->mimeType = 'image/gif';
                imagegif($img);
                break;
            case IMG_JPG:
                $this->mimeType = 'image/jpeg';
                imagejpeg($img, null, $this->quality);
                break;
            case IMG_PNG:
                $this->mimeType = 'image/png';
                imagepng($img, null, $this->quality);
                break;
        }
				*/		
    }

}
